package de.mrbesen.youtubecrawler;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;

import com.mysql.cj.jdbc.exceptions.MysqlDataTruncation;

import de.mrbesen.youtubecrawler.Crawler.Video;

public class DB {

	private Connection con;
	private String server = Config.prop.getProperty("db.host", "localhost"), user = Config.prop.getProperty("db.user", "ytcrawler"), pw = Config.prop.getProperty("db.pw", ""), db = Config.prop.getProperty("db.dbname", "ytcrawler");
	private int port = Integer.parseInt(Config.prop.getProperty("db.port", "3306")); 
	private Logger log = Logger.getLogger(DB.class.getName());


	public DB() {
		try {
			connect(false);

			//set the database up!
			boolean found = false;
			ResultSet set = con.getMetaData().getCatalogs();//does the db exists?
			while(set.next()) {
				if(set.getString(1).equalsIgnoreCase(db)) {
					found = true;
					con.setCatalog(db);
					break;
				}
			}

			if(!found) {//DataBase not found, try to create
				log.warn("Database not found! tring to create!");
				//create DB, table: konten / player / Transactions
				update("CREATE DATABASE `" + db + "` /*!40100 DEFAULT CHARACTER SET latin1*/;");
				con.setCatalog(db);
				update("CREATE TABLE `videos` (`id` varchar(13) NOT NULL,`length` int(11) NOT NULL,`created` int(11) NOT NULL,`langcode` varchar(3) NOT NULL DEFAULT 'en',`category` int(11) DEFAULT NULL, PRIMARY KEY (`id`), UNIQUE KEY `id_UNIQUE` (`id`)) ENGINE=InnoDB DEFAULT CHARSET=latin1;");
				update("CREATE TABLE `temp` ( `ytid` varchar(13) NOT NULL COMMENT 'a Table to store Video ids, when they are found to process them later', PRIMARY KEY (`ytid`), UNIQUE KEY `ytid_UNIQUE` (`ytid`)) ENGINE=InnoDB DEFAULT CHARSET=utf8;");
				
				log.info("Database is set up!");
			}
		} catch (SQLException e) {
			log.error("Error while connecting to the database! ", e);
		}
	}

	private void connect(boolean selectdb) {
		try {
			Class.forName("com.mysql.jdbc.Driver");//Treiber laden  try this driver: com.mysql.cj.jdbc.Driver
			//verbinden
			con = DriverManager.getConnection("jdbc:mysql://" + server + ":" + port + "/" + (selectdb ? db : "") + "?serverTimezone=UTC" ,user,pw);
		}catch (ClassNotFoundException | SQLException e) {
			log.error("Error while connecting to the database! ", e);
		}
	}

	/**
	 * removes all videos, that are known from the db
	 * @param input
	 * @return
	 */
	public List<String> checkvideos(List<String> input) {
		if(!input.isEmpty()) {
			StringBuilder ids  = new StringBuilder();
			for(int i = 0; i < input.size(); i++) {
				ids.append(',').append(input.get(i));
			}
			String query = "SELECT `id` FROM `videos` WHERE concat('%',`id`,'%') LIKE '" + ids.toString() +  "';";
			ResultSet res = query(query);
			try {
				while(res.next()) {
					input.remove(res.getString(1));
				}
			} catch(SQLException e) {
				e.printStackTrace();
			}
		}
		return input;
	}

	/**
	 * save the list of videos to the DB
	 * @param input
	 */
	public void addVideos(List<Video> input) {
		//log.info("add " + input.size() + " videos");
		if(input.size() > 0) {
			StringBuilder sb = new StringBuilder();
			for(int i = 0; i< input.size(); i++) {
				if(i > 0)
					sb.append(',');
				Video v = input.get(i);
				sb.append("('").append(v.id).append("','").append(v.length).append("','").append(v.created).append("','").append(v.languageCode).append("','").append(v.categorie).append("','").append(v.title).append("','").append(v.channel).append("','").append(v.tags).append("') ");
			}
			if(sb.length() > 2) {
				String qu = "INSERT IGNORE INTO `ytcrawler`.`videos`(`id`, `length`, `created`, `langcode`, `category`, `videotitle`, `channel`, `tags`) VALUES " + sb.toString();
				update(qu);
			}
		}
	}

	public void updateVideos(List<Video> input) {
		log.info("Updateing " + input.size() + " videos.");
		for(Video v : input) {
			if(v != null)
				updateVideo(v);
		}
	}

	private void updateVideo(Video v) {
		try {
			String qu = "UPDATE `ytcrawler`.`videos` SET `length` = '" + v.length + "', `created` = '" + v.created + "', `langcode` = SUBSTR('" + v.languageCode + "', 1, 3) ,`category` = '" + v.categorie + "',`videotitle` = SUBSTR('" + v.title + "',1,100),`channel` = SUBSTR('" + v.channel + "',1,20),`tags` = '" + v.tags.substring(0, v.tags.length() > 40 ? 40 : v.tags.length()) + "' WHERE `id` = '" + v.id + "';";
			update(qu);
		} catch(NullPointerException e) {

		}
	}

	public LinkedList<String> getUncompleted(int limit, int offset) {
		LinkedList<String> out = new LinkedList<>();
		String sql = "SELECT `id` FROM `videos` WHERE `channel` IS NULL LIMIT " + offset + "," + limit + ";";
		ResultSet resu = query(sql);
		try {
			while(resu.next()) {
				out.add(resu.getString(1));
			}
		} catch (SQLException e) {
			log.info("error", e);
		}
		return out;
	}

	public void removeVideos(LinkedList<Video> vids) {
		log.info("Delete " + vids.size() + " videos.");
		for(Video s : vids) {
			update("DELETE FROM `ytcrawler`.`videos` WHERE `id`='" + s.id + "';");
		}
	}

	/**
	 * instant query
	 * @param q
	 * @return Das resultSet der Query
	 */

	public ResultSet query(String q) {
		try {
			if(con.isClosed()) {
				connect(true);
			}
			return con.prepareStatement(q).executeQuery();
		} catch (SQLException e) {
			log.error("Fehler bim ausführen der Query: " + q, e);
		}
		return null;//ERROR!
	}

	/**
	 * instant update
	 * @param q
	 */
	public void update(String q) {
		try {
			if(con.isClosed()) {
				connect(true);
			}
			con.prepareStatement(q).executeUpdate();
		} catch (MysqlDataTruncation ignore) {
			log.info("truncated.", ignore);
		} catch (SQLException e) {
			log.error("Fehler bim ausführen der Update-Query: " + q, e);
		}
	}


	public String getRandom() {
		ResultSet set = query("SELECT `id`, rand() as 'r' FROM `videos` ORDER BY r LIMIT 1;");
		if(set == null)
			return null;
		try {
			if(set.next()) {
				return set.getString(1);
			}
		} catch (SQLException e) {
			log.warn("error getting a random video", e);
		}
		return null;
	}

	public LinkedList<String> restoreTemp() {
		ResultSet res = query("SELECT * FROM `ytcrawler`.`temp` LIMIT 0,500;");
		LinkedList<String> out = new LinkedList<>();
		log.info("RestoreTemp");
		try {
			while(res.next()) {
				out.add(res.getString(1));
			}
			update("DELETE FROM `ytcrawler`.`temp` LIMIT 0,500;");
		} catch (Exception e) {}
		return out;
	}

	public void storeTemp(LinkedList<String> strings) {
		if(!strings.isEmpty()) {
			StringBuilder sb = new StringBuilder();
			for(String s : strings) {
				sb.append("'), ('").append(s);
			}
			update("INSERT IGNORE INTO `ytcrawler`.`temp` (`ytid`) VALUES ('" + sb.substring(6).toString() + "');");
		}
	}
}